; PODPROGRAMY OBSUGI INTERFEJSU KART TELEFONICZNYCH (ISO 7816-2)
; Rafa Baranowski (C) 2005
;
; Etykiety procedur:
; 	Ini_KC		- inicjalizacja interfejsu kart
;	Czytaj_KC		- odczyt zawartoci karty
;	KC_wait		- opnienie programowe > 25 us
;
; Wymagane stae:
;	SYS_FREQ 		- czstotliwo pracy w MHz
;	K_KC			- rejestr kierunku portu kart
;	O_KC			- rejestr wyjciowy portu kart
;	I_KC			- rejestr wejciowy portu kart 
;	KC_RS			- nr linii dla sygnau Reset
;	KC_CK			- nr linii dla sygnau Clock
;	KC_IO			- nr linii dla sygnau Input/Output
;

;****************************************************************
;		PROCEDURA INICJALIZUJCA INTERFEJS KARTY
;
Ini_KC:
	push	R16

	in	R16, K_KC
	andi	R16, ~(1<<KC_IO)
	ori	R16, (1<<KC_RS)|(1<<KC_CK)
	out	K_KC, R16		; ustaw kierunki dziaania portw

	in	R16, O_KC
	andi	R16, ~((1<<KC_RS)|(1<<KC_CK))
	ori	R16, 1<<KC_IO
	out	O_KC, R16		; zeruj stan linii CLK i RESET, a
					;   na linii I/O wcz podciganie
	pop	R16
	ret


;****************************************************************
;		PROCEDURA ODCZYTUJCA ZAWARTO KARTY
;
; Parametry:
;	R16	- liczba odczytywanych bajtw
;	X	- wskanik do zmiennej tablicowej (SRAM),
;		  w ktrej zapisana ma zosta zawarto karty
;
.MACRO KC_H				; makro przestawiajce wybrane wyprowadzenie
					;   interfejsu karty w stan wysoki
					; argument: nazwa linii (CK lub RS)
	sbi	O_KC, KC_@0
	rcall	KC_wait
.ENDM

.MACRO KC_L				; makro przestawiajce wybrane wyprowadzenie
					;   interfejsu karty w stan niski
					; argument: nazwa linii (CK lub RS)
	cbi	O_KC, KC_@0
	rcall	KC_wait
.ENDM

Czytaj_KC:
	push	R17
	push	R18

	KC_H	RS
	KC_H	CK
	KC_L	CK
	KC_L	RS			; wstpne zerowanie licznika adresowego karty

	ldi	R17, 8		; zaaduj licznik ptli
Czytaj_KC_1:			; ptla generujca 8 impulsw na linii CK
	KC_H	CK
	KC_L	CK			;   wygeneruj impuls na linii CK
	dec	R17
	brne	Czytaj_KC_1		; zaptl iteracyjnie (x8)

	KC_H	RS
	KC_H	CK
	KC_L	CK
	KC_L	RS			; ostateczne zerowanie licznika adresowego karty
	
Czytaj_KC_2:			; ptla bajtowa
	ldi	R17, 8		;   zaaduj licznik ptli bitowej
Czytaj_KC_2_1:			;   ptla bitowa
	clc
	sbic	I_KC, KC_IO		;     sprawd stan linii I/O
	sec
	rol	R18			;     i wsu go do rejestru R18
	KC_H	CK
	KC_L	CK			;     wygeneruj impuls na linii CK
	dec	R17
	brne	Czytaj_KC_2_1	;   zaptl iteracyjnie (x8)

	st	X+, R18		;   przechowaj odebrany bajt na kolejnej
					;     pozycji tablicy wskazywanej przez X
	dec	R16
	brne	Czytaj_KC_2		; zaptl iteracyjnie dla nastpnego bajtu
		
	pop	R18
	pop	R17
	ret


;****************************************************************
;		PROCEDURA WPROWADZAJCA OPӏNIENIE > 25 us
;
KC_wait:
	push	R16

	ldi	R16, 5*SYS_FREQ-2
KC_wait_1:				; ptla opniajc
	nop
	nop
	dec	R16			; (czas wykonywania 25*SYS_FREQ+2 cykli
	brne	KC_wait_1		;   => 25 us + 2/SYS_FREQ us)

	pop	R16
	ret
