; PODPROGRAMY OBSUGI MYSZY PS/2
; Rafa Baranowski (C) 2005
;
; Etykiety procedur:
; 	Ini_myszy_PS2	- inicjalizacja interfejsu i konfiguracja myszy PS/2
;	Sprawdz_mysz_PS2	- sprawdzanie pozycji i stanu myszy
;	Wyslij_PS2		- przesyanie pojedynczego bajtu
;	Odbierz_PS2		- odbieranie pojedynczego bajtu
;	Zabezpiecz_PS2	- zabezpieczanie interfejsu przed zawieszeniem
;	Odbezpiecz_PS2	- odbezpieczanie interfejsu
;	Blad_PS2		- obsuga bdu myszy (TOV1 int)
;
; Wykorzystywane zasoby:
;	- licznik 1
;
; Wymagane stae:
;	ROZDZ_MYSZY	- rozdzielczo myszy (0...3, 0 - minimum)
;	K_PS2_CLK	- rejestr kierunku dla linii CLK interfejsu
;	O_PS2_CLK	- rejestr wyjciowy dla linii CLK interfejsu
;	I_PS2_CLK	- rejestr wejciowy dla linii CLK interfejsu
;	PS2_CLK	- nr linii dla sygnau CLK
;	K_PS2_DTA	- rejestr kierunku dla linii DTA interfejsu
;	O_PS2_DTA	- rejestr wyjciowy dla linii DTA interfejsu
;	I_PS2_DTA	- rejestr wejciowy dla linii DTA interfejsu
;	PS2_DTA	- nr linii dla sygnau DTA
;
; Wymagane podprogramy dodatkowe:
;	Czekaj_us		- z pliku czekaj_us.inc
;	Czekaj_ms		- z pliku czekaj_ms.inc
;

					; Etykiety rozkazw dla myszy:
.EQU MYSZ_RESET		= 0xFF;   zeruje kontroler myszy
.EQU MYSZ_DEFAULTS	= 0xF6;   wcza ustawienia domylne
.EQU MYSZ_RESOLUTION	= 0xE8;   ustawia rozdzielczo
.EQU MYSZ_REMOTE_MODE	= 0xF0;   wcza tryb odpytywania
.EQU MYSZ_READ_DATA	= 0xEB;   odczytuje stan myszy
					; Etykiety odpowiedzi myszy:
.EQU MYSZ_ACK		= 0xFA;   potwierdzenie odbioru
.EQU MYSZ_PASSED		= 0xAA;   informacja o powodzeniu testu BAT
.EQU MYSZ_ID		= 0x00;   identyfikator standardowej myszy PS/2
.EQU MYSZ_ERROR		= 0xFC;   informacja o bdzie
					; Pozycje bitw w 1. bajcie odbieranym od myszy:
.EQU M_KL_L			= 0	;   bit reprezentujcy stan klawisza lewego
.EQU M_KL_P			= 1	;   bit reprezentujcy stan klawisza prawego
.EQU M_KL_S			= 2	;   bit reprezentujcy stan klawisza rodkowego


.MACRO PS2_czekaj_na_CLK_H	; makro oczekujce na stan wysoki sygnau CLK
	Czekaj:
	sbis	I_PS2_CLK, PS2_CLK
	rjmp	Czekaj
.ENDM

.MACRO PS2_czekaj_na_CLK_L	; makro oczekujce na stan niski sygnau CLK
	Czekaj:
	sbic	I_PS2_CLK, PS2_CLK
	rjmp	Czekaj	
.ENDM


;****************************************************************
;	PROCEDURA INICJALIZUJCA INTERFEJS I KONFIGURUJCA MYSZ PS/2
;
; Wartoci wyjciowe:
;	T (SREG)	- znacznik bdu (T=1 => bd)
;
Ini_myszy_PS2:
	push	R16
	push	R17

	cbi	K_PS2_CLK, PS2_CLK
	cbi	K_PS2_DTA, PS2_DTA; wcz tryb wejciowy na liniach interfejsu

	sbi	O_PS2_CLK, PS2_CLK
	sbi	O_PS2_DTA, PS2_DTA; wcz podciganie na liniach interfejsu

	clr	R16
	out	PLLCSR, R16		; wycz ptl fazow PLL licznika 1
	out	TCCR1A, R16		; wycz funkcje dodatkowe licznika 1
	ldi	R16, (1<<CS13)|(1<<CS12)|(1<<CS11)|(1<<CS10)
	out	TCCR1B, R16		; uruchom licznik 1, taktowany zegarem
					;   systemowym, podzielonym w stosunku 1:16384
					;   (dla 1 MHz przepenienie co ok. 4 s)
	ldi	R16, 100
	ldi	R17, 1
	rcall	Czekaj_ms		; odczekaj 100 ms na wypadek, gdyby mysz bya
					;   w trakcie inicjalizacji po wczeniu zasilania

	ldi	R16, MYSZ_RESET
	rcall	Wyslij_PS2		; przelij rozkaz inicjalizacji
	brts	Ini_myszy_blad	;   jeli wystpi bd transmisji, to go sygnalizuj

	rcall	Odbierz_PS2		; odbierz 1. bajt (spodziewane potwierdzenie) 
	brts	Ini_myszy_blad	;   jeli wystpi bd transmisji, to go sygnalizuj
	cpi	R16, MYSZ_ACK	;   jeli nie odebrano potwierdzenia, to
	brne	Ini_myszy_blad	;     sygnalizuj bd

	rcall	Odbierz_PS2		; odbierz 2. bajt (spodziewana inf. o powodzeniu BAT)
	brts	Ini_myszy_blad	;   jeli wystpi bd transmisji, to go sygnalizuj
	cpi	R16, MYSZ_PASSED	;   jeli nie odebrano inf. o powodzeniu BAT, to
	brne	Ini_myszy_blad	;     sygnalizuj bd

	rcall	Odbierz_PS2		; odbierz 3. bajt (spodziewany identyfikator)
	brts	Ini_myszy_blad	;   jeli wystpi bd transmisji, to go sygnalizuj
	cpi	R16, MYSZ_ID	;   jeli nie odebrano poprawnego identyfikatora, to
	brne	Ini_myszy_blad	;     sygnalizuj bd

	ldi	R16, MYSZ_REMOTE_MODE
	rcall	Wyslij_PS2		; przelij rozkaz wczenia trybu odpytywania
	brts	Ini_myszy_blad	;   jeli wystpi bd transmisji, to go sygnalizuj

	rcall	Odbierz_PS2		; odbierz bajt (spodziewane potwierdzenie)
	brts	Ini_myszy_blad	;   jeli wystpi bd transmisji, to go sygnalizuj
	cpi	R16, MYSZ_ACK	;   jeli nie odebrano potwierdzenia, to
	brne	Ini_myszy_blad	;     sygnalizuj bd

	ldi	R16, MYSZ_RESOLUTION
	rcall	Wyslij_PS2		; przelij rozkaz nastawy rozdzielczoci
	brts	Ini_myszy_blad	;   jeli wystpi bd transmisji, to go sygnalizuj

	rcall	Odbierz_PS2		; odbierz bajt (spodziewane potwierdzenie)
	brts	Ini_myszy_blad	;   jeli wystpi bd transmisji, to go sygnalizuj
	cpi	R16, MYSZ_ACK	;   jeli nie odebrano potwierdzenia, to
	brne	Ini_myszy_blad	;     sygnalizuj bd

	ldi	R16, ROZDZ_MYSZY
	rcall	Wyslij_PS2		; przelij kod proponowanej rozdzielczoci
	brts	Ini_myszy_blad	;   jeli wystpi bd transmisji, to go sygnalizuj

	rcall	Odbierz_PS2		; odbierz bajt (spodziewane potwierdzenie)
	brts	Ini_myszy_blad	;   jeli wystpi bd transmisji, to go sygnalizuj
	cpi	R16, MYSZ_ACK	;   jeli nie odebrano potwierdzenia, to
	brne	Ini_myszy_blad	;     sygnalizuj bd

	clt				; zeruj umowny znacznik bdu
	pop	R17
	pop	R16
	ret

Ini_myszy_blad:			; obsuga bdu:
	set				;   ustaw umowny znacznik bdu
	pop	R17
	pop	R16
	ret


;****************************************************************
;	PROCEDURA SPRAWDZAJCA AKTUALN POZYCJ I STAN MYSZY
;
; Wartoci wyjciowe:
;	R16		- warto przesunicia wzgldnego w osi Y
;	R17		- warto przesunicia wzgldnego w osi X
;	R18		- stan klawiszy (b2 - rodkowy; b1 - prawy; b0 - lewy)
;	T (SREG)	- znacznik bdu (T=1 => bd)
;
Sprawdz_mysz_PS2:
	push	R19

	ldi	R16, MYSZ_READ_DATA
	rcall	Wyslij_PS2		; przelij rozkaz raportowania stanu
	brts	Spr_mysz_blad	;   jeli wystpi bd transmisji, to go sygnalizuj
	rcall	Odbierz_PS2		; odbierz 1. bajt (spodziewane potwierdzenie) 
	brts	Spr_mysz_blad	;   jeli wystpi bd transmisji, to go sygnalizuj
	cpi	R16, MYSZ_ACK	;   jeli nie odebrano potwierdzenia, to
	brne	Spr_mysz_blad	;     sygnalizuj bd
	rcall	Odbierz_PS2		; odbierz 2. bajt (m.in. stan klawiszy)
	mov	R18, R16		;   i zapamitaj go w R18
	brts	Spr_mysz_blad	;   jeli wystpi bd transmisji, to go sygnalizuj
	rcall	Odbierz_PS2		; odbierz 3. bajt (przemieszczenie X)
	mov	R17, R16		;   i zapamitaj go w R17
	brts	Spr_mysz_blad	;   jeli wystpi bd transmisji, to go sygnalizuj
	rcall	Odbierz_PS2		; odbierz 4. bajt (przemieszczenie Y)
					;   i zapamitaj go w R16
	brts	Spr_mysz_blad	;   jeli wystpi bd transmisji, to go sygnalizuj

	andi	R18, (1<<M_KL_L)|(1<<M_KL_P)|(1<<M_KL_S)
					; w rejestrze stanu myszy pozostaw tylko
					;   bity informujce o stanie klawisza

	clt				; zeruj umowny znacznik bdu
	pop	R19
	ret

Spr_mysz_blad:			; obsuga bdu:
	clr	R16
	clr	R17
	clr	R18			;   zeruj wartoci wyjciowe
	set				;   i ustaw umowny znacznik bdu
	pop	R19
	ret


;****************************************************************
;		PROCEDURA PRZESYAJCA POJEDYNCZY BAJT
;
; Parametry:
;	R16		- warto przesyana (kod rozkazu)
;
; Wartoci wyjciowe:
;	T (SREG)	- znacznik bdu (T=1 => bd)
;
Wyslij_PS2:
	push	R17
	push	R18

	rcall	Bit_kontr_PS2	; oblicz bit kontrolny wartoci przesyanej (R16)
					;   i zapisz go w znaczniku T (SREG)

	mov	R17, R16		; skopiuj warto przesyan do R17

	ldi	R16, 10
	rcall	Czekaj_us		; odczekaj ok. 100 us

	sbi	K_PS2_DTA, PS2_DTA; przestaw lini DTA w tryb wyjciowy
	sbi	K_PS2_CLK, PS2_CLK; przestaw lini CLK w tryb wyjciowy

	cbi	O_PS2_CLK, PS2_CLK; wymu niski stan na linii CLK

	ldi	R16, 15
	rcall	Czekaj_us		; odczekaj ok. 150 us

	cbi	O_PS2_DTA, PS2_DTA; wymu niski stan na linii DTA

	sbi	O_PS2_CLK, PS2_CLK; zwolnij lini CLK 
	cbi	K_PS2_CLK, PS2_CLK; i przestaw j w tryb wejciowy

	rcall	Zabezpiecz_PS2	; zabezpiecz interfejs przed zawieszeniem

	PS2_czekaj_na_CLK_H	; czekaj kolejno na zbocze narastajce
	PS2_czekaj_na_CLK_L	; i opadajce na linii CLK

	bld	R16, 0
	lsr	R16			; skopiuj znacznik T do C w SREG (bit kontrolny)
	ldi	R16, 9		; inicjalizacja licznika ptli wykonywanej 9 razy
Wyslij_PS2_1:
	ror	R17
	cbi	O_PS2_DTA, PS2_DTA;   wyprowad na lini DTA kolejny bit wartoci
	brcc	Wyslij_PS2_2	;     nadawanej w kolejnoci LSB...MSB
	sbi	O_PS2_DTA, PS2_DTA;     (a w 9. cyklu - bit kontrolny)
Wyslij_PS2_2:
	PS2_czekaj_na_CLK_H	;   czekaj kolejno na zbocze narastajce
	PS2_czekaj_na_CLK_L	;   i opadajce CLK (przesy kolejnego bitu)
	dec	R16
	brne	Wyslij_PS2_1	; zaptl iteracyjne (x9)

	sbi	O_PS2_DTA, PS2_DTA; zwolnij lini DTA
	cbi	K_PS2_DTA, PS2_DTA; i przestaw j w tryb wejciowy

	PS2_czekaj_na_CLK_H	; czekaj kolejno na zbocze narastajce
	PS2_czekaj_na_CLK_L	; i opadajce CLK (przesy bitu stopu)
	clt
	sbic	I_PS2_DTA, PS2_DTA; zapisz w znaczniku T ze SREG odebrany
	set				;   bit potwierdzenia

	PS2_czekaj_na_CLK_H	; czekaj na ostatnie zbocze narastajce CLK

	rcall	Odbezpiecz_PS2	; odbezpiecz interfejs

	pop	R18
	pop	R17
	ret


;****************************************************************
;		PROCEDURA ODBIERAJCA POJEDYNCZY BAJT
;
; Wartoci wyjciowe:
;	R16		- warto odebrana
;	T (SREG)	- znacznik bdu (T=1 => bd)
;
Odbierz_PS2:
	push	R17
	push	R18

	rcall	Zabezpiecz_PS2	; zabezpiecz interfejs przed zawieszeniem

	PS2_czekaj_na_CLK_L	; czekaj na zbocze opadajce CLK
					;   (odbir bitu startu)

	sbic	I_PS2_DTA, PS2_DTA; jeli nie odebrano bitu startu (DTA=1), to
	rjmp	Odb_PS2_blad	;   przejd do procedury obsugi bdu

	clr	R16			; inicjalizacja zmiennej wartoci odbieranej
	ldi	R17, 8		; inicjalizacja licznika ptli wykonywanej 8 razy
Odb_PS2_1:
	PS2_czekaj_na_CLK_H	;   czekaj kolejno na zbocze narastajce
	PS2_czekaj_na_CLK_L	;   i opadajce CLK (odbir kolejnego bitu)
	clc
	sbic	I_PS2_DTA, PS2_DTA;   zapisz aktualny stan linii DTA do znacznika
	sec				;     C z rejestru SREG
	ror	R16			;   i wsu go do rejestru wartoci odbieranej
	dec	R17
	brne	Odb_PS2_1		; zaptl iteracyjne (x8)
	
	PS2_czekaj_na_CLK_H	; czekaj na kolejne zbocze narastajce
	PS2_czekaj_na_CLK_L	; i opadajce CLK (odbir bitu kontrolnego)
	in	R17, I_PS2_DTA
	bst	R17, PS2_DTA	; zapisz bit kontrolny w znaczniku T ze SREG
	
	PS2_czekaj_na_CLK_H	; czekaj na kolejne zbocze narastajce
	PS2_czekaj_na_CLK_L	; i opadajce CLK (odbir bitu stopu)
	sbis	I_PS2_DTA, PS2_DTA; jeli nie odebrano bitu stopu (DTA=0), to
	rjmp	Odb_PS2_blad	;   przejd do procedury obsugi bdu

	clr	R17			; na najmodszej pozycji R17 zapisz
	bld	R17, 0		;   odebrany bit kontrolny

	rcall	Bit_kontr_PS2	; oblicz bit kontrolny dla wartoci odebranej (R16)

	clr	R18			; na najmodszej pozycji R18 zapisz
	bld	R18, 0		;   obliczony bit kontrolny

	eor	R17, R18		; jeli odebrany bit kontrolny nie zgadza si
	brne	Odb_PS2_blad	;   z odebranym, to przejd do obsugi bdu

	PS2_czekaj_na_CLK_H	; czekaj na zwolnienie linii CLK

	rcall	Odbezpiecz_PS2	; odbezpiecz interfejs
	clt				; wyzeruj umowny znacznik bdu
	pop	R18
	pop	R17
	ret

Odb_PS2_blad:			; procedura obsugi bdu	
	rcall	Odbezpiecz_PS2	;   odbezpiecz interfejs
	set				;   i ustaw umowny znacznik bdu
	pop	R18
	pop	R17
	ret


;****************************************************************
;	PROCEDURA OBLICZAJCA BIT KONTROLNY TYPU ODD
;
; Parametry:
;	R16		- warto podlegajca kontroli
;
; Wartoci wyjciowe:
;	T (SREG)	- obliczony bit kontrolny typu odd
;
Bit_kontr_PS2:
	push	R16
	push	R17
	push	R18

	clr	R17			; inicjalizacja "licznika jedynek"
	ldi	R18, 8		; inicjalizacja licznika ptli wyk. 8 razy
B_kontr_PS2_1:
	lsr	R16			;   przesu warto kontrolowan
	brcc	B_kontr_PS2_2	;   jeli wysunito "jedynk", to
	inc	R17			;     zwiksz licznik jedynek
B_kontr_PS2_2:
	dec	R18
	brne	B_kontr_PS2_1	; zaptl iteracyjne (x8)

	com	R17
	bst	R17, 0		; zapamitaj bit kontrolny w znaczniku T

	pop	R18
	pop	R17
	pop	R16
	ret


;****************************************************************
;	PROCEDURA ZABEZPIECZAJCA INTERFEJS PRZED ZAWIESZENIEM
;
Zabezpiecz_PS2:
	push	R16

	clr	R16
	out	TCNT1, R16		; zeruj zawarto licznika 1

	ldi	R16, 1<<TOV1
	out	TIFR, R16		; zeruj ewentualne danie przerwania TOV1

	in	R16, TIMSK
	ori	R16, 1<<TOIE1
	out	TIMSK, R16		; zezwl na przerwania TOV1

	sei				; wcz obsug przerwa

	pop	R16
	ret


;****************************************************************
;		PROCEDURA ODBEZPIECZAJCA INTERFEJS
;
Odbezpiecz_PS2:
	push	R16

	in	R16, TIMSK
	andi	R16, ~(1<<TOIE1)
	out	TIMSK, R16		; zablokuj obsug przerwa TOV1

	pop	R16
	ret


;****************************************************************
;	PROCEDURA OBSUGI ZAWIESZENIA INTERFEJSU (obsuga TOV1 int)
;
Blad_PS2:
	in	R16, TIMSK
	andi	R16, ~(1<<TOIE1)
	out	TIMSK, R16		; zablokuj obsug przerwa TOV1

	pop	R16			; cignij ze stosu lad do przerwanej
	pop	R16			;   procedury (Wyslij_PS2 lub Odbierz_PS2)

	pop	R18			; cignij ze stosu zawartoci R18 i R17,
	pop	R17			;   ktre umiecia na nim przerwana procedura

	set				; ustaw umowny znacznik bdu
	reti				; i powr do programu, ktry wywoa
					;   przerwan procedur
