; Przestrajalny generator przebiegu cyfrowego
; Rafa Baranowski (C) 2005
;
; Fizyczna konfiguracja mikrokontrolera:
;	PB3...0	- wyjcie generowanego przebiegu (Y)
;	PA7...0	- wejcie przestrajajce czstotliwo (X)
;	PB6		- wejcie sygnau strobujcego (S)

					; SEGMENT DEKLARACJI
.INCLUDE "tn26def.inc"		; docz plik nagwkowy dla ATtiny26

					; deklaracje symbolicznych nazw rejestrw f.:
.EQU	K_Y	= DDRB		;   rejestr kierunku wyjcia Y
.EQU	O_Y	= PORTB		;   rejestr stanu wyjcia Y
.EQU	I_X	= PINA		;   rejestr stanu wejcia X


.CSEG					; SEGMENT PROGRAMU

.ORG 0x0000				; wektor zerowania
rjmp	Reset
.ORG INT0addr			; wektor przerwania zewntrznego INT0
rjmp	Zmien_czestotliwosc

					; deklaracja tablicy staych (pami programu),
					;   ktr tworz kolejne wartoci pewnego
					;   cyfrowego przebiegu okresowego
Tabl:					;   (wartoci z przedziau 0...15)
	.DB	0, 	13,	1,	10,	0,	1,	5,	5
	.DB	3,	8,	15,	14,	4,	2,	2,	6
	.DB	10,	8,	5,	4,	1,	13,	8,	14
	.DB	6,	10,	9,	5,	4,	15,	0xFF, 0xFF
					; tablica zakoczona jest wartociami 0xFF,
					;   ktre posu do wykrycia koca sekwencji

Reset:
	ldi	R16, RAMEND		; inicjalizacja wskanika stosu
	out	SP, R16

	in	R16, MCUCR
	ori	R16, (1<<ISC01)|(1<<ISC00)
	out	MCUCR, R16		; narastajce zbocze na S (PB6 / INT0) powoduje
					;   wyzwolenie przerwania zewntrznego INT0

	in	R16, GIMSK
	ori	R16, (1<<INT0)
	out	GIMSK, R16		; odblokowanie przerwania zewntrznego INT0	

	sei				; uaktywnienie globalnego zezwolenia na przerwania

	ldi	R16, 0x0F		; ustawienie modszego pbajtu w R16
	out	K_Y, R16		; modsza cz portu PB pracuje jako wyjcie (Y)

	ldi	R16, (1<<CTC1)|(1<<CS12)|(1<<CS11)|(1<<CS10)
	out	TCCR1B, R16		; licznik 1 pracuje w trybie CTC, zliczajc
					;   z czstotliwoci fosc/64
					;   (czstotliwo przepenie zalena bdzie od
					;	 wartoci X z portu PA)
Inicjalizacja_Z:
	ldi	ZH, high(Tabl<<1)	; wskanik Z wskazuje na poczatek zdeklarowanej
	ldi	ZL, low(Tabl<<1)	;   tablicy staych w pamici programu

Petla:				; ptla gwna programu
	lpm	R16, Z		; aduj R16 wartoci staej z pamici programu,
					;   wskazywanej przez rejestr indeksowy Z
	adiw	ZL, 1			;   i inkrementuj wskanik Z
	cpi	R16, 0xFF		; porwnaj odczytan sta z wartoci 0xFF
	brne	Petla_dalej		;   jeli rne, kontynuuj od etykiety "Petla_dalej"
	rjmp	Inicjalizacja_Z	;   jeli rwne, zainicjalizuj ponownie Z, bo
					;     natrafiono na koniec tablicy
Petla_dalej:
	out	O_Y, R16		; wyprowad na wyjcie Y (port PB) warto staej

Petla_czekaj:			; ptla oczekujca na zdarzenie zgodnego porwnania
	in	R16, TIFR		;   wartoci licznika 1 z zawartoci rejestru
	sbrs	R16, TOV1		;   OCR1C i automatyczne zerowanie licznika
	rjmp	Petla_czekaj	;   (oczekiwanie na TOV1=1)

	ldi	R16, 1<<TOV1
	out	TIFR, R16		; zerowanie znacznika przepenienia TOV1

	rjmp	Petla			; zaptl program (skocz do etykiety "Petla")


Zmien_czestotliwosc:	; podprogram obsugi przerwania INT0 - uruchamiany,
				;   gdy na wejciu S (PB6) wystpi zbocze narastajce;
				;   odpowiada temu bdzie zmiana czstotliwoci
				;	przepenie licznika 1 w zalenoci od X

	in	R18, SREG	; odczytaj zawarto SREG sprzed przerwania
	push	R18		; umie na stosie zawarto SREG

	in	R18, I_X	; odczytaj warto X (z portu PA)
	com	R18		; zaneguj bity wartoci X
	out	OCR1C, R18	; zmie czstotliwo przepenie licznika 1
				;   zgodnie z odczytan wartoci X
				;   (im wiksza warto X, tym mniejsze OCR1C i
				;    tym czstsze przepenienia)
	clr	R18
	out	TCNT1, R18	; zeruj licznik 1

	pop	R18		; cignij ze stosu zawarto SREG sprzed
	out	SREG, R18	;   przerwania i wpisz t warto do SREG
	reti
