; Najprostszy generator funkcyjny, obsugiwany mysz PS/2
; Rafa Baranowski (C) 2005

.INCLUDE "tn26def.inc"		; SEGMENT DEKLARACJI

.EQU SYS_FREQ	= 16		; czstotliwo systemowa w MHz
					; stae generatora
.EQU K_GEN		= DDRB	;   rejestr kierunku portu
.EQU O_GEN		= PORTB	;   rejestr wyjciowy portu
.EQU GEN_SYG	= 1		;   nr linii dla sygnau wyjciowego (OC1A/PB1)
					; stae interfejsu PS/2 i myszy
.EQU ROZDZ_MYSZY	= 1		;   rozdzielczo myszy (0...3, 0 - minimum)
.EQU K_PS2_CLK	= DDRA	;   rejestr kierunku dla linii CLK
.EQU O_PS2_CLK	= PORTA	;   rejestr wyjciowy dla linii CLK
.EQU I_PS2_CLK	= PINA	;   rejestr wejciowy dla linii CLK
.EQU PS2_CLK	= 7		;   nr linii dla sygnau CLK
.EQU K_PS2_DTA	= DDRB	;   rejestr kierunku dla linii DTA
.EQU O_PS2_DTA	= PORTB	;   rejestr wyjciowy dla linii DTA
.EQU I_PS2_DTA	= PINB	;   rejestr wejciowy dla linii DTA
.EQU PS2_DTA	= 6		;   nr linii dla sygnau DTA

					; nazwy symboliczne rejestrw roboczych
.DEF USTAW_TYP	= R10		;   rejestr typu generowanego sygnau (0...3)
.DEF USTAW_TLUM	= R12		;   rejestr stopnia tumienia sygnau (0...8)
.DEF USTAW_OFFSET	= R13		;   rejestr skadowej staej sygnau (-64...63)
.DEF USTAW_OKRES	= R14		;   rejestr okresu generacji (1...5)
.DEF USTAW_KROK	= R15		;   rejestr kroku generacji (1/2/4/8/16/32/64/128)


.CSEG					; SEGMENT PROGRAMU
.ORG	0				; wektor zerowania
	rjmp	Reset			;   skok do procedury inicjalizujcej
.ORG	OVF0addr			; wektor przerwania TOV0
	rjmp	Generacja		;   skok do procedury wystawiajcej kolejn prbk

Reset:				; podprogram inicjalizujcy
	ldi	R16, RAMEND
	out	SP, R16		; inicjalizacja wskanika stosu

	clr	USTAW_TYP		; ustal zerowy typ sygnau (sinusoida)
	clr	USTAW_TLUM		; ustal zerowy stopie tumienia sygnau
	clr	USTAW_OFFSET	; ustal zerow skadow sta sygnau
	ldi	R16, 1
	mov	USTAW_OKRES, R16	; ustal najmniejszy okres generacji
	mov	USTAW_KROK, R16	; ustal najmniejszy krok generacji

	ldi	ZL, low(Tab_sinus<<1)	; zaaduj wskanik Z adresem pierwszej
	ldi	ZH, high(Tab_sinus<<1)	;   prbki w tablicy sygnau sinusoidalnego

	ldi	R16, 1<<PLLE
	out	PLLCSR, R16		; odblokuj ptl fazow (PLL)
Res_czekaj_na_PLL:
	in	R16, PLLCSR
	sbrs	R16, PLOCK		; oczekuj na ustabilizowanie si PLL
	rjmp	Res_czekaj_na_PLL
	ldi	R16, (1<<PLLE)|(1<<PCKE)
	out	PLLCSR, R16		; wykorzystaj sygna ptli PLL (PCK - 64 MHz)
					;   do taktowania licznika 1
	ldi	R16, (1<<CS10)
	out	TCCR1B, R16		; uruchom licznik 1, taktowany z cz. 64 MHz			
	ldi	R16, (1<<COM1A1)|(1<<PWM1A)
	out	TCCR1A, R16		; przecz licznik 1 w tryb PWM i wcz
					;    generacj sygnau PWM na linii OC1A/PB1
	ldi	R16, 0x7F		; ustal pojemno licznika 1 na 128
	out	OCR1C, R16		;   (czstotliwo sygnau PWM rwna 500 kHz) 
	sbi	K_GEN, GEN_SYG	; wcz tryb wyjciowy na linii OC1A/PB1

	out	TCCR0, USTAW_OKRES; uruchom licznik 0, taktowany z cz. 16 MHz
	ldi	R16, (1<<TOIE0)
	out	TIMSK, R16		; odblokuj obsug przerwa TOV0

	rcall	Ini_myszy_PS2	; inicjalizacja myszy PC

	sei				; wcz obsug przerwa

Petla:				; ptla gwna programu
	rcall Sprawdz_mysz_PS2	; sprawd stan myszy; teraz:
					;   R18 - zawiera stan przyciskw
					;   R17 - zawiera przemieszczenie X
					;   R16 - zawiera przemieszczenie Y

	tst	R18			; jeli aden klawisz nie zosta nacinity,
	breq	Petla			;   to powr do ptli gwnej

Ptl_klaw_L:
	sbrs	R18, M_KL_L		; jeli nacinito lewy klawisz myszy, to...
	rjmp	Ptl_klaw_P

	mov	R21, USTAW_OFFSET
	mov	R22, USTAW_TLUM

	clr	R19			;   inicjalizuj akumulator przesuni w osi X
	clr	R20			;   inicjalizuj akumulator przesuni w osi Y
Ptl_klaw_L_1:
	neg	R17
	add	R19, R17		;   sumuj przesunicia myszy w osi X
	brvc	Ptl_klaw_L_2	;   i zadbaj, by akumulator R19 si nie przepeni
	ldi	R19, 127
	brmi	Ptl_klaw_L_2
	ldi	R19, -128
Ptl_klaw_L_2:
	add	R20, R16		;   sumuj przesunicia myszy w osi Y
	brvc	Ptl_klaw_L_off	;   i zadbaj, by akumulator R20 si nie przepeni
	ldi	R20, 127
	brmi	Ptl_klaw_L_off
	ldi	R20, -128

Ptl_klaw_L_off:			;   na podstawie przesunicia w osi Y oblicz
	mov	R16, R21		;     zmian zawartoci rejestru okrelajcego
	add	R16, R20		;     warto skadowej staej sygnau (USTAW_OFFSET)
	brvc	Ptl_klaw_L_off_1	;   i zadbaj, by warto USTAW_OFFSET zawieraa
	ldi	R16, 63		;     si w zakresie -64...63
	brmi	Ptl_klaw_L_off_1
	ldi	R16, -64
Ptl_klaw_L_off_1:
	cpi	R16, 64
	brlt	Ptl_klaw_L_off_2
	ldi	R16, 63
Ptl_klaw_L_off_2:
	cpi	R16, -64
	brge	Ptl_klaw_L_off_3
	ldi	R16, -64
Ptl_klaw_L_off_3:
	cpi	R16, -5
	brlt	Ptl_klaw_L_off_4	;   jeli zawarto USTAW_OFFSET jest wiksze ni -5
	cpi	R16, 6
	brge	Ptl_klaw_L_off_4  ;   i mniejsze ni 5, to...
	clr	R16			;     wyzeruj USTAW_OFFSET (dziki temu nastawa
Ptl_klaw_L_off_4:			;     zerowego przesnicia bdzie atwiejsza)
	mov	USTAW_OFFSET, R16

Ptl_klaw_L_tlum:			;   na podstawie przesunicia w osi X oblicz
	mov	R16, R22		;     zmian zawartoci rejestru okrelajcego
	mov	R17, R19		;     tumienie sygnau (USTAW_TLUM)
	asr	R17
	asr	R17
	asr	R17
	inc	R17
	asr	R17
	add	R16, R17
	brpl	Ptl_klaw_L_tlum_1	;   i zadbaj, by warto USTAW_TLUM zawieraa 
	clr	R16			;     si w zakresie 0...8
Ptl_klaw_L_tlum_1:
	cpi	R16, 9
	brlo	Ptl_klaw_L_tlum_2
	ldi	R16, 8
Ptl_klaw_L_tlum_2:
	mov	USTAW_TLUM, R16

	rcall	Sprawdz_mysz_PS2	;   ponownie sprawd stan myszy
	tst	R18
	sbrc	R18, M_KL_P		;   jeli nacinito klawisz prawy, to...
	rjmp	Ptl_klaw_S		;     wykonaj procedur obsugi klawisza rodkowego
	brne	Ptl_klaw_L_1	;   jeli natomiast wszystkie kl. zostay zwolnione,
	rjmp	Petla			;     to powr do ptli gwnej programu


Ptl_klaw_P:
	sbrs	R18, M_KL_P		; jeli nacinito prawy klawisz myszy, to...
	rjmp	Ptl_klaw_S

	mov	R21, USTAW_OKRES
	mov	R22, USTAW_KROK

	clr	R19			;   inicjalizuj akumulator przesuni w osi X
	clr	R20			;   inicjalizuj akumulator przesuni w osi Y
Ptl_klaw_P_1:
	add	R19, R17		;   sumuj przesunicia myszy w osi X
	brvc	Ptl_klaw_P_2	;   i zadbaj, by akumulator R19 si nie przepeni
	ldi	R19, 127
	brmi	Ptl_klaw_P_2
	ldi	R19, -128
Ptl_klaw_P_2:
	neg	R16
	add	R20, R16		;   sumuj przesunicia myszy w osi Y
	brvc	Ptl_klaw_P_okr	;   i zadbaj, by akumulator R20 si nie przepeni
	ldi	R20, 127
	brmi	Ptl_klaw_P_okr
	ldi	R20, -128

Ptl_klaw_P_okr:			;   na podstawie przesunicia w osi Y oblicz
	mov	R16, R21		;     zmian zawartoci rejestru okrelajcego
	mov	R17, R20		;     okres generacji (USTAW_OKRES)
	asr	R17
	asr	R17
	asr	R17
	inc	R17
	asr	R17
	add	R16, R17
	cpi	R16, 1
	brge	Ptl_klaw_P_okr_1	;   i zadbaj, by warto USTAW_OKRES zawieraa
	ldi	R16, 1		;     si w zakresie 1...5
Ptl_klaw_P_okr_1:
	cpi	R16, 6
	brlo	Ptl_klaw_P_okr_2
	ldi	R16, 5
Ptl_klaw_P_okr_2:
	mov	USTAW_OKRES, R16
	out	TCCR0, USTAW_OKRES;   wykorzystaj warto USTAW_OKRES do nastawy
					;     wspczynnika podziau czstotliwoci
					;     taktowania licznika 0

Ptl_klaw_P_krok:			;   na podstawie przesunicia w osi X oblicz
	mov	R16, R22		;     zmian zawartoci rejestru okrelajcego
	mov	R17, R19		;     krok generacji (USTAW_KROK)
	asr	R17
	asr	R17
	asr	R17
	inc	R17
	asr	R17
	breq	Ptl_klaw_P_krok_5
	brmi	Ptl_klaw_P_krok_3
Ptl_klaw_P_krok_1:
	lsl	R16
	brcc	Ptl_klaw_P_krok_2	;   i zadbaj, by warto USTAW_KROK bya
	ror	R16			;     jedn z dodatnich potg liczby 2
	rjmp	Ptl_klaw_P_krok_5
Ptl_klaw_P_krok_2:
	dec	R17
	brne	Ptl_klaw_P_krok_1
	rjmp	Ptl_klaw_P_krok_5
Ptl_klaw_P_krok_3:
	lsr	R16
	brcc	Ptl_klaw_P_krok_4
	rol	R16
	rjmp	Ptl_klaw_P_krok_5
Ptl_klaw_P_krok_4:
	inc	R17
	brne	Ptl_klaw_P_krok_3
Ptl_klaw_P_krok_5:
	cp	USTAW_KROK, R16
	breq	Ptl_klaw_P_3	;   jeli warto USTAW_KROK si zmienia, 
	mov	USTAW_KROK, R16	;     to zapamitaj j
	clr	ZL			;     i wyzeruj LSB wskanika Z

Ptl_klaw_P_3:
	rcall	Sprawdz_mysz_PS2	;   ponownie sprawd stan myszy
	tst	R18
	sbrc	R18, M_KL_L		;   jeli nacinito klawisz lewy, to...
	rjmp	Ptl_klaw_S		;     wykonaj procedur obsugi klawisza rodkowego
	brne	Ptl_klaw_P_1	;   jeli natomiast wszystkie kl. zostay zwolnione,
	rjmp	Petla			;     to powr do ptli gwnej programu


Ptl_klaw_S:				; jeli nacinito rodkowy klawisz myszy, to...
	inc	USTAW_TYP		;   zwiksz zawarto rejestru okrelajcego
	ldi	R16, 4		;     typ generowanego sygnau (USTAW_TYP)
	cp	USTAW_TYP, R16
	brlo	Ptl_klaw_S_1	;   i zadbaj, by zawarto USTAW_TYP znajdowaa
	clr	USTAW_TYP		;     si w zakresie 0..3
Ptl_klaw_S_1:
	ldi	ZH, high(Tab_sinus<<1)
	add	ZH, USTAW_TYP	;   zaaduj wskanik Z adresem tej tablicy sygnau,
					;     ktra jest okrelona przez warto USTAW_TYP
	clr	ZL			;   i wyzeruj LSB wskanika Z
Ptl_klaw_S_2:
	rcall Sprawdz_mysz_PS2	; ponownie sprawd stan myszy
	andi	R18, (1<<M_KL_L)|(1<<M_KL_P)|(1<<M_KL_S)
	tst	R18
	brne	Ptl_klaw_S_2	; i oczekuj na zwolnienie klawiszy
	rjmp	Petla			; a nastpnie powr do wykonywania ptli gwnej


;*******************************************************************************
Generacja:				; PROCEDURA OBSUGI PRZERWANIA TOV0
	push	R16
	push	R17
	in	R16, SREG
	push	R16

	in	R16, TCNT0
	ori	R16, 0x80		; ustaw tak zawarto licznika 1, aby jego kolejne
	out	TCNT0, R16		;   przepenienie nastpio po 128 cyklach licznikowych

	add	ZL, USTAW_KROK	; inkrementuj LSB wskanika do tablicy sygnau,
					;   biorc pod uwag krok okrelony przez USTAW_KROK
	lpm	R16, Z		; odczytaj kolejn prbk wartoci sygnau

	mov	R17, USTAW_TLUM
	tst	R17
	breq	Gen_2
Gen_1:				; dziel warto nowej prbki przez liczb 2 tyle razy,
	asr	R16			;    ile wynosi zawarto rejestru USTAW_TLUM
	dec	R17
	brne	Gen_1
Gen_2:
	add	R16, USTAW_OFFSET	; zwiksz warto prbki o skadow sta,
					;   okrelon przez zawarto rejestru USTAW_OFFSET
	cpi	R16, 64		; i zadbaj, by warto prbki znajdowaa si w
	brlt	Gen_3			;   zakresie -64...63
	ldi	R16, 63
	rjmp	Gen_4
Gen_3:
	cpi	R16, -64
	brge	Gen_4
	ldi	R16, -64
Gen_4:
	subi	R16, -64		; zwiksz warto prbki o poow zakresu
	out	OCR1A, R16		; zaaduj warto prbki do rejestru OCR1A,
					;   okrelajcego wypenienie przebiegu PWM
					;   na wyprowadzeniu OC1A/PB1
	pop	R16
	out	SREG, R16
	pop	R17
	pop	R16
	reti

					; DOCZANE PODPROGRAMY
.INCLUDE "czekaj_ms.inc"	;   oczekiwanie milisekundowe
.INCLUDE "czekaj_us.inc"	;   oczekiwanie mikrosekundowe
.INCLUDE "mysz_ps2_bz.inc"	;   obsuga myszy PS/2 (bez zabezpieczenia)


.ORG	FLASHEND-4*128+1
;*******************************************************************************
Tab_sinus:				; tablica 256 wartoci funkcji sinusoidalnej 
.DB 63, 63, 63, 63, 63, 63, 62, 62, 62, 61, 61, 61, 60, 60, 59, 59
.DB 58, 58, 57, 56, 56, 55, 54, 53, 52, 52, 51, 50, 49, 48, 47, 46
.DB 45, 43, 42, 41, 40, 39, 38, 36, 35, 34, 32, 31, 30, 28, 27, 26
.DB 24, 23, 21, 20, 18, 17, 15, 14, 12, 11, 9, 8, 6, 5, 3, 2
.DB 0, -2, -3, -5, -6, -8, -9, -11, -12, -14, -15, -17, -18, -20, -21, -23
.DB -24, -26, -27, -28, -30, -31, -32, -34, -35, -36, -38, -39, -40, -41, -42, -43
.DB -45, -46, -47, -48, -49, -50, -51, -52, -52, -53, -54, -55, -56, -56, -57, -58
.DB -58, -59, -59, -60, -60, -61, -61, -61, -62, -62, -62, -63, -63, -63, -63, -63
.DB -63, -63, -63, -63, -63, -63, -62, -62, -62, -61, -61, -61, -60, -60, -59, -59
.DB -58, -58, -57, -56, -56, -55, -54, -53, -52, -52, -51, -50, -49, -48, -47, -46
.DB -45, -43, -42, -41, -40, -39, -38, -36, -35, -34, -32, -31, -30, -28, -27, -26
.DB -24, -23, -21, -20, -18, -17, -15, -14, -12, -11, -9, -8, -6, -5, -3, -2
.DB 0, 2, 3, 5, 6, 8, 9, 11, 12, 14, 15, 17, 18, 20, 21, 23
.DB 24, 26, 27, 28, 30, 31, 32, 34, 35, 36, 38, 39, 40, 41, 42, 43
.DB 45, 46, 47, 48, 49, 50, 51, 52, 52, 53, 54, 55, 56, 56, 57, 58
.DB 58, 59, 59, 60, 60, 61, 61, 61, 62, 62, 62, 63, 63, 63, 63, 63
;*******************************************************************************
Tab_trojkat:			; tablica 256 wartoci funkcji trjktnej
.DB -64, -63, -62, -61, -60, -59, -58, -57, -56, -55, -54, -53, -52, -51, -50, -49
.DB -48, -47, -46, -45, -44, -43, -42, -41, -40, -39, -38, -37, -36, -35, -34, -33
.DB -32, -31, -30, -29, -28, -27, -26, -25, -24, -23, -22, -21, -20, -19, -18, -17
.DB -16, -15, -14, -13, -12, -11, -10, -9, -8, -7, -6, -5, -4, -3, -2, -1
.DB 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15
.DB 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31
.DB 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47
.DB 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63
.DB 63, 62, 61, 60, 59, 58, 57, 56, 55, 54, 53, 52, 51, 50, 49, 48
.DB 47, 46, 45, 44, 43, 42, 41, 40, 39, 38, 37, 36, 35, 34, 33, 32
.DB 31, 30, 29, 28, 27, 26, 25, 24, 23, 22, 21, 20, 19, 18, 17, 16
.DB 15, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0
.DB -1, -2, -3, -4, -5, -6, -7, -8, -9, -10, -11, -12, -13, -14, -15, -16
.DB -17, -18, -19, -20, -21, -22, -23, -24, -25, -26, -27, -28, -29, -30, -31, -32
.DB -33, -34, -35, -36, -37, -38, -39, -40, -41, -42, -43, -44, -45, -46, -47, -48
.DB -49, -50, -51, -52, -53, -54, -55, -56, -57, -58, -59, -60, -61, -62, -63, -64
;*******************************************************************************
Tab_pila:				; tablica 256 wartoci funkcji pioksztatnej
.DB -64, -64, -63, -63, -62, -62, -61, -61, -60, -60, -59, -59, -58, -58, -57, -57
.DB -56, -56, -55, -55, -54, -54, -53, -53, -52, -52, -51, -51, -50, -50, -49, -49
.DB -48, -48, -47, -47, -46, -46, -45, -45, -44, -44, -43, -43, -42, -42, -41, -41
.DB -40, -40, -39, -39, -38, -38, -37, -37, -36, -36, -35, -35, -34, -34, -33, -33
.DB -32, -32, -31, -31, -30, -30, -29, -29, -28, -28, -27, -27, -26, -26, -25, -25
.DB -24, -24, -23, -23, -22, -22, -21, -21, -20, -20, -19, -19, -18, -18, -17, -17
.DB -16, -16, -15, -15, -14, -14, -13, -13, -12, -12, -11, -11, -10, -10, -9, -9
.DB -8, -8, -7, -7, -6, -6, -5, -5, -4, -4, -3, -3, -2, -2, -1, -1
.DB 0, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7
.DB 8, 8, 9, 9, 10, 10, 11, 11, 12, 12, 13, 13, 14, 14, 15, 15
.DB 16, 16, 17, 17, 18, 18, 19, 19, 20, 20, 21, 21, 22, 22, 23, 23
.DB 24, 24, 25, 25, 26, 26, 27, 27, 28, 28, 29, 29, 30, 30, 31, 31
.DB 32, 32, 33, 33, 34, 34, 35, 35, 36, 36, 37, 37, 38, 38, 39, 39
.DB 40, 40, 41, 41, 42, 42, 43, 43, 44, 44, 45, 45, 46, 46, 47, 47
.DB 48, 48, 49, 49, 50, 50, 51, 51, 52, 52, 53, 53, 54, 54, 55, 55
.DB 56, 56, 57, 57, 58, 58, 59, 59, 60, 60, 61, 61, 62, 62, 63, 63
;*******************************************************************************
Tab_prost:				; tablica 256 wartoci funkcji prostoktnej
.DB 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63
.DB 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63
.DB 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63
.DB 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63
.DB 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63
.DB 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63
.DB 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63
.DB 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63
.DB -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64
.DB -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64
.DB -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64
.DB -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64
.DB -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64
.DB -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64
.DB -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64
.DB -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64, -64
