; PODPROGRAMY OBSUGI ODBIORNIKA PODCZERWIENI (KODOWANIE SIRC)
; Rafa Baranowski (C) 2005
;
; Wykorzystywane zasoby:
;	- licznik 0
;	- modu przerwa zewntrznych INT0
;	- 3 bajty w pamici SRAM
;
; Etykiety procedur:
; 	Ini_IR	- inicjalizacja odbiornika podczerwieni
;	Odbior_IR	- odbir wiadomoci SIRC (obsuga INT0)
;			  (uwaga: w trakcie jej wykonywania SREG(I)=1)
;
; Wymagane stae:
;	OSC_FREQ	- czstotliwo rezonatora kwarcowego [Hz]
;	K_IR_IN	- rejestr kierunku portu odbiornika 
;	O_IR_IN	- rejestr wyjciowy portu odbiornika
;	I_IR_IN	- rejestr wejciowy portu odbiornika
;	IR_IN		- nr linii dla sygnau odbieranego (INT0)
;
; Wymagane zmienne:
;	IR_rozkaz	- zmienna bajtowa w SRAM (rozkaz wiadomoci SIRC)
;	IR_adres	- zmienna bajtowa w SRAM (adres wiadomoci SIRC)
;	IR_znacz	- zmienna bajtowa w SRAM (znaczniki wiadomoci)
;


;****************************************************************
;	PROCEDURA INICJALIZUJCA ODBIORNIK PODCZERWIENI
;
Ini_IR:
	push	R16
	
	cbi	K_IR_IN, IR_IN	; linia IR_IN pracuje jako wejcie
	sbi	O_IR_IN, IR_IN	; z aktywnym podciganiem

	in	R16, MCUCR
	sbr	R16, 1<<ISC01
	cbr	R16, 1<<ISC00
	out	MCUCR, R16		; przerwanie INT0 wywoywane zboczem opadajcym

	ldi	R16, 1<<INTF0
	out	GIFR, R16		; zeruj znacznik dania przerwania INT0

	in	R16, GIMSK
	sbr	R16, 1<<INT0
	out	GIMSK, R16		; wcz obsug przerwania INT0

	in	R16, TIMSK
	cbr	R16, 1<<TOIE0
	out	TIMSK, R16		; wycz obsug przerwa dla licznika 0

	ldi	R16, (1<<CS01)|(1<<CS00)
	out	TCCR0, R16		; wcz licznik 0, taktowany z czstotliwoci
					;   systemow podzielon w stosunku 1:64
	clr	R16
	sts	IR_znacz, R16
	sts	IR_rozkaz, R16
	sts	IR_adres, R16	; zeruj wykorzystywane zmienne w pamici SRAM

	pop	R16
	ret


;****************************************************************
;	PROCEDURA ODBIERAJCA WIADOMO W KODZIE SIRC (obsuga INT0)
;
; Wartoci wyjciowe:
;	IR_rozkaz	- rozkaz odebranej wiadomoci SIRC (SRAM)
;	IR_adres 	- adres odebranej wiadomoci SIRC (SRAM)
;	IR_znacz	- znacznik wiadomoci SIRC (SRAM)
;			  (jeli odebrano now wiadomo, bit IR_NOWY
;			  tej zmiennej przyjmuje stan wysoki)
;

.EQU IR_NOWY = 0	; nr znacznika nowej wiadomoci w zmiennej IR_znacz

; *****************************
.MACRO IR_laduj_licznik		; makro adujce licznik 0 i zerujce TOV0
					; argument: czas, po jakim nastpi ma
					;           przepenienie licznika [ms]
	ldi	R16, 256-INT((@0)*OSC_FREQ/64000 + 0.5)
	rcall	IR_laduj_licznik_1
.ENDM

IR_laduj_licznik_1:		; procedura pomocnicza dla powyszego makra
	out	TCNT0, R16
	ldi	R16, 1<<TOV0
	out	TIFR, R16
	ret
; *****************************
.MACRO IR_czekaj_na_TOV0	; makro oczekujce na przepenienie licznika 0
	Czekaj:
	in	R16, TIFR
	sbrs	R16, TOV0
	rjmp	Czekaj
.ENDM
; *****************************
.MACRO IR_czekaj_na_H		; makro oczekujce na stan wysoki sygnau IR
	Czekaj:			;   jeli sygna nie przyjmie wartoci
	in	R16, TIFR		;   wysokiej przed przepenieniem licznika 0,
	sbrc	R16, TOV0		;   to procedura odbierajca zostanie przerwana
	rjmp	Odb_IR_koniec
	sbis	I_IR_IN, IR_IN
	rjmp	Czekaj
.ENDM
; *****************************
.MACRO IR_czekaj_na_L		; makro oczekujce na stan niski sygnau IR
	Czekaj:			;   jeli sygna nie przyjmie wartoci
	in	R16, TIFR		;   niskiej przed przepenieniem licznika 0,
	sbrc	R16, TOV0		;   to procedura odbierajca zostanie przerwana
	rjmp	Odb_IR_koniec
	sbic	I_IR_IN, IR_IN
	rjmp	Czekaj
.ENDM
; *****************************
Odbior_IR:				; procedura odbierajca wiadomo SIRC
	push	R16
	push	R17
	push	R18
	push	R19
	in	R16, SREG
	push	R16

	in	R16, GIMSK
	cbr	R16, 1<<INT0
	out	GIMSK, R16		; tymczasowo wycz obsug przerwania INT0

	sei				; przywr globaln obsug przerwa

	IR_laduj_licznik 0.9
	IR_czekaj_na_TOV0
	IR_laduj_licznik 0.9
	IR_czekaj_na_TOV0		; zaczekaj 1.8 ms (0.9+0.9 ms)

	sbic	I_IR_IN, IR_IN	; jeli sygna nie znajduje si ju w stanie
	rjmp	Odb_IR_koniec	;   niskim, najprawdopodobniej nie odebrano
					;   bitu startu, wic zakocz procedur odbioru
	IR_laduj_licznik 0.6+0.3
	IR_czekaj_na_H		; czekaj do koca bitu startu (planowo 0.6 ms)

	clr	R18			; inicjalizacja zmiennej 7-bitowego rozkazu
	ldi	R17, 7		; inicjalizacja licznika ptli wyk. 7 razy
Odb_IR_rozkaz_ptl:
	IR_laduj_licznik 0.6+0.3
	IR_czekaj_na_L		;   czekaj do koca przerwy (planowo 0.6 ms)

	IR_laduj_licznik 0.9
	IR_czekaj_na_TOV0		;   czekaj okoo 0.9 ms

	clc

	sbic	I_IR_IN, IR_IN	;   jeli sygna jest ju w stanie wysokim, to...
	rjmp	Odb_IR_rozkaz_ok	;     odebrano bit o wartoci niskiej -> C=0
					;   w przeciwnym razie...
	sec				;     odebrano bit o wartoci wysokiej -> C=1

	IR_laduj_licznik 0.3+0.3
	IR_czekaj_na_H		;     poczekaj do jego koca (planowo 0.3 ms)

Odb_IR_rozkaz_ok:
	ror	R18			;   wsu odebrany bit do rejestru rozkazu	
	dec	R17
	brne	Odb_IR_rozkaz_ptl	; zaptl iteracyjnie (x7)
	lsr	R18			; skoryguj rozkaz, przesuwajc go o 1 pozycj

	clr	R19			; inicjalizacja zmiennej 5-bitowego adresu
	ldi	R17, 5		; inicjalizacja licznika ptli wyk. 5 razy
Odb_IR_adres_ptl:
	IR_laduj_licznik 0.6+0.3
	IR_czekaj_na_L		;   czekaj do koca przerwy (planowo 0.6 ms)

	IR_laduj_licznik 0.9
	IR_czekaj_na_TOV0		;   czekaj okoo 0.9 ms

	clc

	sbic	I_IR_IN, IR_IN	;   jeli sygna jest ju w stanie wysokim, to...
	rjmp	Odb_IR_adres_ok	;     odebrano bit o wartoci niskiej -> C=0
					;   w przeciwnym razie...
	sec				;     odebrano bit o wartoci wysokiej -> C=1

	IR_laduj_licznik 0.3+0.3
	IR_czekaj_na_H		;     poczekaj do jego koca (planowo 0.3 ms)

Odb_IR_adres_ok:
	ror	R19			;   wsu odebrany bit do rejestru adresu
	dec	R17
	brne	Odb_IR_adres_ptl	; zaptl iteracyjnie (x7)
	lsr	R19
	lsr	R19
	lsr	R19			; skoryguj adres, przesuwajc go o 3 pozycje

	ldi	R16, 1<<IR_NOWY
	sts	IR_znacz, R16	; ustaw znacznik nowej wiadomoci w zm. IR_znacz
	sts	IR_rozkaz, R18	; zapisz odebrany rozkaz w zmiennej IR_rozkaz
	sts	IR_adres, R19	; zapisz adres wiadomoci w zmiennej IR_adres

Odb_IR_koniec:
	ldi	R16, 1<<INTF0
	out	GIFR, R16		; anuluj ewentualne danie przerwania INT0

	in	R16, GIMSK
	sbr	R16, 1<<INT0
	out	GIMSK, R16		; przywr obsug przerwania INT0

	pop	R16
	out	SREG, R16
	pop	R19
	pop	R18
	pop	R17
	pop	R16
	reti
